/* Encyklopedie jazyka C
 *
 * Kapitola Moduly a knihovny
 *
 * 2023-08-12-20.00
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#define VERZE "2.1"
#define NAZEV "Moduly a knihovny"

// TODO Prostuduj si obsah tohoto hlavičkového souboru! Jsou tam zaklínadla,
// bez kterých některé věci ve Windows nefungují správně!
// - barevný výstup
// - printg - český tisk (překódovává z UTF8 na CP852)
// - printh - český tisk + vyprávěcí mód
// - formátovací a formátové značky pro printf/scanf podle normy
// - obsluha nabídek v kapitolách
#include "pomocny-modul.h"

// Implementace nabídkového menu
#include "menu.h"

#include <stdio.h>
#include <stdlib.h>
// #include <string.h>
#include <stdbool.h>
// #include <ctype.h>
// #include <math.h>
// #include <float.h>
// #include <limits.h>


// Hlavičky funkcí realizující jednotlivé kapitoly.

void coJeToModul(void);
void vyuzitiModulu(void);
void radyProTvorbuModulu(void);

void hlavickoveSoubory(void);
void zdrojoveSoubory(void);
void prekladProgramuSModuly(void);

void coJeToKnihovna(void);
void prekladKnihovny(void);
//void test(void);


int main(void)
{
  povolBarevnyVystup();
//  prepniRychlostVypraveni(2);

  Tmenu * menu = menuVytvor(
    NADPIS(
      "==============================================================\n"
      "Encyklopedie jazyka C - " NAZEV " (verze " VERZE ")\n"
      "(c) David Martinek, 2023\n"
      "Zároveň s programem sleduj i zdrojový text!\n"
      "==============================================================\n"
    )
    , zobrazMod
  );

  if (menu == NULL)
  {
    printg("Není dost paměti ani pro výrobu menu. Končím program.");
    return 0;
  }

  menuPridej(menu,   0, "\n" SEKCE("Moduly obecně") , NULL);
  menuPridej(menu, 'A', "Co je to modul\n", coJeToModul);
  menuPridej(menu, 'B', "Využití modulů\n", vyuzitiModulu);
  menuPridej(menu, 'C', "Rady pro tvorbu modulů\n", radyProTvorbuModulu);

  menuPridej(menu,   0, "\n" SEKCE("Moduly v jazyce C") , NULL);
  menuPridej(menu, 'D', "Hlavičkové soubory\n", hlavickoveSoubory);
  menuPridej(menu, 'E', "Zdrojové soubory\n", zdrojoveSoubory);
  menuPridej(menu, 'F', "Překlad programu s moduly\n", prekladProgramuSModuly);

  menuPridej(menu,   0, "\n" SEKCE("Knihovny") , NULL);
  menuPridej(menu, 'G', "Co je to knihovna\n", coJeToKnihovna);
  menuPridej(menu, 'H', "Překlad a použití knihovny\n", prekladKnihovny);

//  menuPridej(menu,   0, "\n" SEKCE("Nadpis") , NULL);
//  menuPridej(menu, 'I', "\n", test);
//  menuPridej(menu, 'J', "\n", test);
//  menuPridej(menu, 'K', "\n", test);
//  menuPridej(menu, 'L', "\n", test);

  menuPridej(menu,   0, "\n" SEKCE("Nastavení programu"), NULL);
  menuPridej(menu, '0', "Nastav vyprávěcí mód\n", nastavRychlostVypraveni);

  while (menuVolba(menu) != 'Q') {}

  printg("Končím program.\n");
  menuZrus(menu);
  return 0;
}



////////////////////////////////////////////////////////////////////////////////
void coJeToModul(void)
{
  zacatekKapitoly("Co je to modul");

  printh(
    "- Modul je zásobárna opakovatelně použitelných typů, konstant, funkcí\n"
    "  a případně i maker, které řeší příbuzné problémy.\n"
    "- Lze jej používat opakovaně v různých projektech.\n"
    "- Slouží ke zvýšení komfortu. => Není třeba opakovaně \"vymýšlet kolo\".\n"
    "\n"
    "- Z modulů lze skládat jak běžné programy, tak knihovny.\n"
    "\n"
  );

  if (navratNeboSekce("Části modulu")) return;
  if (pokracovat())
  {
    printh(
      "- Modul je tvořen převážně kódem funkcí, které chceme využívat v jiných\n"
      "  částech programu.\n"
      "- Kromě nich obsahuje i pomocné funkce, které není žádoucí používat mimo něj.\n"
      "- Rozhraní modulu slouží ke specifikaci, co uživatel modulu smí používat\n"
      "  nebo co je pro něj viditelné.\n"
      "\n"
      "- Jazyk C používá pro specifikaci rozhraní hlavičkové soubory (.h).\n"
      "- Zde se nacházejí pouze deklarace veřejných funkcí (hlavičky, tedy\n"
      "  prototypy), typů a konstant.\n"
      "\n"
      "- Implementace veřejných i pomocných funkcí se v C nacházejí v samostatném\n"
      "  zdrojovém souboru (.c).\n"
      "- Ke každému hlavičkovému souboru skoro vždy existuje alespoň jeden zdrojový.\n"
      // Výjimkou jsou hlavičkové soubory, které neobsahují žádné prototypy
      // funkcí. Mohou ale obsahovat deklarace konstant a typů, případně makra.
      "\n"
    );
  }

  if (navratNeboSekce("Jaké moduly mají smysl")) return;
  if (pokracovat())
  {
    printh(
      "- Modul by měly tvořit datové typy, konstanty a  podprogramy,\n"
      "  které k sobě nějak logicky logicky patří.\n"
      "- Typicky jsou to nástroje pro řešení jednoho typu problémů.\n"
      "- Příkladem je uspořádání modulů standardní knihovny jazyka C:\n"
      "  - stdio.h   - funkce pro práci se vstupem a výstupem\n"
      "  - string.h  - funkce pro práci se stringy\n"
      "  - ctype.h   - makra a funkce pro testování kategorií znaků\n"
      "  - stdbool.h - typ " KK("bool") " a konstanty " KK("true") " a " KK("false") ".\n"
      "  - stdlib.h  - funkce pro běžnou potřebu a práci s pamětí\n"
      "  - math.h    - matematické funkce\n"
      "  - float.h   - konstanty pro popis racionálních typů (minima, maxima apod.)\n"
      "  - limits.h  - konstanty pro popis celočíselných typů\n"
      "\n"
      "- V objektově orientovaných jazycích obvykle jeden modul obsahuje jednu\n"
      "  třídu.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Moduly v C a jiných jazycích")) return;
  if (pokracovat())
  {
    printh(
      "- V jazyce C je modul obvykle tvořen jedním zdrojovým (.c) a jedním\n"
      "  hlavičkovým (.h) souborem. (Může jich ale být i více.)\n"
      "- Jazyk C nemá žádné syntaktické konstrukce pro tvorbu modulů.\n"
      "- Využívá preprocesor a oddělený překlad modulů kompilátorem.\n"
      "- Preprocesor je program, který se spouští vždy před kompilátorem\n"
      "  a dělá textové náhrady ve zdrojovém textu. Viz kapitolu Preprocesor.\n"
      "- Použití modulu se v C dělá pomocí příkazu preprocesoru " KK("#include") ",\n"
      "  kterým se v místě použití modulu vloží jeho hlavičkový soubor.\n"
      "\n"
      "- Jazyk C++ řeší moduly stejně jako jazyk C.\n"
      "\n"
//      "- V Javě se moduly zároveň organizují do podadresářů a tvoří takzvané\n"
//      "  balíky (klíčové slovo " KK("package") ").\n"
//      "- Python a jiné interpretované jazyky dělají moduly pouhým rozdělením\n"
//      "  zdrojového textu do více souborů.\n"
//      "- Objektově orientované jazyky typicky rozhraní modulů včetně práv\n"
//      "  přístupu specifikují na úrovni tříd.\n"
//      "  => Nepoužívají hlavičkové soubory ani preprocesor.\n"
      "- Jiné jazyky mohou mít pro práci s modulem speciální sadu příkazů.\n"
//      "- Příkladem jsou jazyky jako Pascal, Object Pascal nebo Eiffel, které\n"
//      "  mají klíčová slova pro tvorbu modulů a navíc umožňují provádět při\n"
//      "  zavádění modulu nějaký inicializační kód.\n"
      "- U novějších jazyků je obvyklé, že nepoužívají preprocesor.\n"
      "- Příkazy jako " KK("include") " nebo " KK("import") " bývají přímo\n"
      "  součástí jazyka.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}


////////////////////////////////////////////////////////////////////////////////
void vyuzitiModulu(void)
{
  zacatekKapitoly("Využití modulů");

  printh(
    "- Moduly používají všechny běžné programovací jazyky.\n"
    "\n"
    "- Je to základní nástroj (nejen v jazyce C) pro tvorbu jakéhokoli SW.\n"
    "- Moduly nepoužívají jen velice triviální programy.\n"
    "  => Je-li tvůj program větší než 500 řádků, začni používat moduly.\n"
    "     Je to snadné a tvůj program bude vypadat profesionálněji.\n"
    // Údaj o 500 řádcích ber s rezervou, ale přibližně od takové velikosti
    // projektu už stojí za to přemýšlet o tvorbě samostatných modulů.
    // Čím dříve programátor toto rozhodnutí udělá, tím méně práce s tím pak je.
    // Moduly totiž programátora přinutí myslet jinak
    // - plánovat budoucí použití,
    // - vytvářet podprogramy obecněji,
    // - zamýšlet se nad bezpečností navržených podprogramů.
    "\n"
  );

  if (navratNeboSekce("Modul jako nástroj zapouzdření")) return;
  if (pokracovat())
  {
    printh(
      "- Pojem \"zapouzdření\" je známý z objektově orientovaného programování.\n"
      "- Vyjadřuje schopnost mít pohromadě typy a operace, které k sobě logicky\n"
      "  patří a případně schopnost řídit k nim přístup (veřejné/skryté funkce).\n"
      "- Tuto schopnost má i obyčejný modul.\n"
      "- Vše, co je v hlavičkovém souboru, se stává součástí veřejného rozhraní.\n"
      "- To, co je deklarováno pouze ve zdrojovém textu, je privátní a viditelné\n"
      "  jen uvnitř modulu samého.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Výhody používání modulů")) return;
  if (pokracovat())
  {
    printh(
      PODSEKCE("Přehlednost")
      "- Modul zvyšuje přehlednost programu.\n"
      "- Funkce a typy, které logicky patří k sobě jsou společně v menším\n"
      "  zdrojovém souboru.\n"
      "\n"
      PODSEKCE("Pohodlí")
      "- Používání modulů zvětšuje pohodlí při práci na projektu.\n"
      "- Při tvorbě modulu se soustředíme na menší objem příbuzných problémů.\n"
      "- Vidět, že určité části projektu jsou již hotové, je uspokojující.\n"
      // Naopak bývá velice frustrující, když projekt tvoří obrovský zdrojový
      // soubor a pořád to nefunguje. => Dělá to dojem, že je to celé rozbité.
      "\n"
      PODSEKCE("Bezpečnost")
      "- Program s moduly bývá bezpečnější.\n"
      "- Samostatné moduly se snadněji testují. Lze si k nim naspat testovací\n"
      "  aplikaci. \n"
      "- Existují knihovny pro tzv. test-driven development (TDD):\n"
      "  CUnit, Unity, Check, Ceedling, Google Test, Criterion, ...\n"
      "\n"
      PODSEKCE("Znovupoužitelnost")
      "- Modul jde použít v různých projektech, což šetří práci.\n"
      "- Potřeba obecných řešení omezuje používání klonovaného kódu.\n"
      // Takový ten nešvar, kdy studenti raději vyrobí pět totožných funkcí,
      // které se liší třeba jen jediným porovnávacím operátorem, místo aby
      // napsali obecnější řešení.
      // Samotný modul samozřejmě takovým prasárnám nezabrání, ale podle
      // zkušeností práce s moduly lidi vede k obecnějšímu přemýšlení nad
      // problémy.
      "- Chyby se pak v programech projevují konzistentně, což usnadňuje jejich\n"
      "  hledání.\n"
      "- Znovupoužitelnost je také důvod pro tvorbu knihoven funkcí.\n"
      "\n"
      PODSEKCE("Týmová spolupráce")
      "- Větší projekty tvoří týmy vývojářů.\n"
      "- Moduly přirozeně usnadňují rozdělení práce mezi vývojáře.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Nevýhody modulů")) return;
  if (pokracovat())
  {
    printh(
      "- Obecně výhody modulů převažují nad nevýhodami.\n"
      "- Problémy modulů vyplývají z jejich nevhodného návrhu nebo použití.\n"
      "\n"
      PODSEKCE("Složité závislosti")
      "- Příliš mnoho modulů může vytvořit spletitý systém závislostí.\n"
      "- To se pak špatně spravuje.\n"
      "\n"
      PODSEKCE("Nevhodný návrh rozhraní")
      "- Pokud jsou moduly navrženy tak špatně, že to vyžaduje změny rozhraní,\n"
      "  je potřeba dělat úpravy všude, kde byl modul použit.\n"
      "=> Rada: Mysli na to už v době návrhu.\n"
      "\n"
      PODSEKCE("Nároky na paměť")
      "- Nevhodné používání modulů a knihoven zvyšuje nároky na paměť.\n"
      "- Příkladem je zbytečné natahování knihoven a modulů, které vlastně\n"
      "  nejsou potřeba.\n"
      // Typickým příkladem je import balik.*; v Javě nebo from modul import *
      // v Pythonu.
      "\n"
      PODSEKCE("Doba překladu")
      "- Příliš mnoho miniaturních modulů znamená prodloužení doby překladu.\n"
      "- V C/C++ se každý modul překládá samostatně.\n"
      "  => Mnoho modulů znamená mnoho spuštěných instancí překladače.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void radyProTvorbuModulu(void)
{
  zacatekKapitoly("Programátorské rady pro tvorbu modulů");

  printh(
    "- Používání modulů zvyšuje kvalitu programu, když se používají správně.\n"
    "- Špatně navržené moduly ale mohou učinit program velmi špatně udržovatelným.\n"
    "\n"
    "- Aby se projevily výhody modulů, stačí se řídit několika pravidly.\n"
    "- Odměnou bude mnohem větší pohodlí při práci na projektu.\n"
    "\n"
  );

  if (navratNeboSekce("Robustní veřejné funkce")) return;
  if (pokracovat())
  {
    printh(
      "+ Veřejné funkce musí být robustní.\n"
      "+ Musí pečlivě ošetřovat vstupní parametry.\n"
      "+ Na chybné hodnoty musí reagovat přesně definovaným a popsaným způsobem.\n"
      "+ Obvykle veřejná funkce ošetřuje parametry a pak volá nějakou vnitřní\n"
      "  neveřejnou funkci, která řeší algoritmus.\n"
      // Říká se tomu princip obalovací funkce (wrapper function).
      "\n"
      "- Zavolání veřejné funkce s jakýmkoli vstupem nesmí vést k havárii.\n"
      "  => Ne vždy se toho dá docílit, ale je třeba se o to snažit.\n"
      // Příkladem jsou funkce zpracovávající ukazatel (třeba na otevřený
      // soubor). Tam jde testovat, zda parametr není NULL, ale nejde otestovat,
      // zda nejde o neinicializovaný ukazatel odkazující na náhodné místo
      // v paměti.
      // Někdy chceme, aby byly funkce efektivní a příliš mnoho kontrol by vedlo
      // k výraznému zpomalení programu. Pak ale musí být vytvořena velice
      // důkladná dokumentace. I tak to obvykle bývá pouhé provizorium, než
      // někdo vymyslí řešení, které bude bezpečné a efektivní zároveň.
      "\n"
    );
  }

  if (navratNeboSekce("Dokumentační komentáře")) return;
  if (pokracovat())
  {
    printh(
      "+ V hlavičkovém souboru tvoří většinu textu dokumentační komentáře.\n"
      "+ Ideální je formát pro doxygen použitý i v tomto projektu.\n"
      "+ Dokumentace popisuje, co to dělá, jak to reaguje na různé vstupy,\n"
      "  co to vrací a kdy je to ne/vhodné používat.\n"
      "+ Dokumentace musí být smysluplná.\n"
      "+ Dokumentace se píše ještě před tím, než se začne programovat!\n"
      // Popsáním toho, jak se bude funkce chovat a jak bude reagovat na různé
      // vstupy se vlastně prování analýza a návrh funkce. Když se programuje
      // dříve, než se začne nad problémem přemýšlet, musí se to obvykle
      // několikrát předělávat. Tohle platí pro jakýkoli programovací jazyk
      // a vlastně pro jakoukoli inženýrskou činnost!
      "\n"
      "- Dokumentační komentáře překládající jazyk C do češtiny, nemají smysl.\n"
      // Nemá smysl popisovat, co dělá každý řádek kódu.
      "\n"
      "- Ukázka prototypu funkce s dokumentačním komentářem:\n"
      "\n"
      KOD(
      "/** Vytiskne zadanou osobu na zadaný výstup.\n"
      " *\n"
      " * \\param f FILE* Otevřený výstupní soubor nebo datový proud.\n"
      " * \\param o Tosoba Osoba, která se bude tisknou.\n"
      " */\n"
      "void tiskniOsobu(FILE * f, Tosoba o);\n"
      )
      "\n"
      "- Tento formát umí zpracovat program doxygen do tvaru HTML stránek.\n"
      // Pokud máte program doxygen nainstalován ve svém systému, můžete jej
      // používat i v Code::Blocks pomocí menu DoxyBlocks.
//      "- Podobný formát zavedl a dodnes používá jazyk Java, který používá\n"
//      "  program javadoc ke stejnému účelu jako doxygen.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Co do modulu dát a co ne")) return;
  if (pokracovat())
  {
    printh(
      "+ Modul musí jít výstižně pojmenovat.\n"
      "- Pokud to nejde, asi obsahuje věci, které do něj nepatří.\n"
      "+ Do modulu patří typy, funkce a konstanty, které řeší stejnou třídu problémů.\n"
      "- Modul nesmí být smetiště, kam se náhodně odklidí věci, co jinde překáží.\n"
      "- V hlavičkovém souboru nesmí být žádný kód.\n"
      // Dělalo by to problémy při linkování - viz podkapitola o hlavičkových
      // souborech.
      "\n"
    );
  }

  konecKapitoly(__func__);
}




//////////////////////////////////////////////////////////////////////////////
void hlavickoveSoubory(void)
{
  zacatekKapitoly("Hlavičkový soubor modulu");

  printh(
    "- Hlavičkový soubor slouží jako veřejné rozhraní modulu.\n"
    "- Jeho obsah se vkládá příkazem preprocesoru " KK("#include") " do všech zdrojových\n"
    "  textů, kde je potřeba používat funkce, které modul nabízí.\n"
    "\n"
  );


  if (navratNeboSekce("Ukázka - modul.h")) return;
  if (pokracovat())
  {
    printh(
      "- Prostuduj si i samotný zdrojový text souboru modul.h.\n"
      "- Je opatřen dokumentačními komentáři.\n"
      "\n"
      KOD(
      "#ifndef MODUL_H_INCLUDED\n"
      "#define MODUL_H_INCLUDED\n"
      "\n"
      "#include <stdio.h>\n"
      "\n"
      "#define SYMBOLICKA_KONSTANTA 12345\n"
      "\n"
      "extern const int KONSTANTNI_PROMENNA;\n"
      "\n"
      "typedef struct _osoba {\n"
      "  const char * jmeno;\n"
      "  unsigned int vek;\n"
      "} Tosoba;\n"
      "\n"
      "Tosoba vyrobOsobu(const char *jmeno, int rokNarozeni);\n"
      "void tiskniOsobu(FILE * f, Tosoba o);\n"
      "\n"
      "#endif // MODUL_H_INCLUDED\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka použití modulu modul.h")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  #include \"modul.h\"\n"
      "\n"
      "  printf(\"  => SYMBOLICKA_KONSTANTA: %%d\\n\", SYMBOLICKA_KONSTANTA);\n"
      "  printf(\"  => KONSTANTNI_PROMENNA: %%d\\n\", KONSTANTNI_PROMENNA);\n"
      "  printf(\"  => osoba:\\n\");\n"
      "  Tosoba osoba = vyrobOsobu(\"Adam\", 2017);\n"
      "  tiskniOsobu(stdout, osoba);\n"
      )
      "\n"
    );

    // Tohle se obvykle dělá na začátku zdrojového souboru, ale jde to i později.
    // V tomto případě budou veškeré deklarace omezeny na tento blok kódu.
    #include "modul.h"

    printf("  => SYMBOLICKA_KONSTANTA: %d\n", SYMBOLICKA_KONSTANTA);
    printf("  => KONSTANTNI_PROMENNA: %d\n", KONSTANTNI_PROMENNA);
    printf("  => osoba:\n");
    Tosoba osoba = vyrobOsobu("Adam", 2017);
    tiskniOsobu(stdout, osoba);

    printh(
      "\n"
      "- Implementační část modulu je probrána v další kapitole.\n"
      "\n"
    );
  }


  if (navratNeboSekce("Hlavičkový soubor")) return;
  if (pokracovat())
  {
    printh(
      "- Obsahuje výhradně deklarace\n"
      "  - hlaviček funkcí - prototypů, blíže viz kapitolu Funkce,\n"
      "  - symbolických konstant,\n"
      "  - globálních proměnných (typicky konstantních),\n"
      "  - datových typů,\n"
      "  - a maker.\n"
      "- Nesmí se zde objevit žádná definice proměnné, ani implementace funkce.\n"
      "  To patří do zdrojového souboru.\n"
      "- Vše, co je zde deklarováno je veřejným rozhraním modulu.\n"
      "\n"
      "- Musí mít vloženo rozhraní modulů, jejichž typy a deklarace jsou v něm\n"
      "  použity. V naší ukázce je to " KK("stdio.h") ".\n"
      // Pozor! Nevkládat zbytečnosti - prodlužuje to dobu překladu.
      "\n"
      "- Protože jde o veřejné rozhraní, měly by být všechny deklarace pečlivě\n"
      "  opatřeny dokumentačními komentáři.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Použití hlavičkového souboru")) return;
  if (pokracovat())
  {
    printh(
      "- Rozlišují se systémové a lokální hlavičkové soubory.\n"
      "- Systémové jsou nainstalovány v systému spolu se systémovými knihovnami.\n"
      "- Lokální se očekávají v adresáři aktuálního projektu.\n"
      "\n"
      "- Rozhraní se natahují pomocí příkazu preprocesoru " KK("#include") ".\n"
      "- Tento příkaz zajistí, že se před překladem programu na toto místo vloží\n"
      "  kompletní obsah zadaného souboru.\n"
      "\n"
      KOD(
      "  #include <systemovy_modul.h>\n"
      "  #include \"lokalni_modul.h\"\n"
      )
      "\n"
      "- U systémových hlavičkových souborů se používají úhlové závorky.\n"
      "- Pro natažení lokálních souborů se používají uvozovky.\n"
      "- Název souboru může obsahovat i cestu - typicky relativní k výchozímu\n"
      "  adresáři.\n"
      // Výchozí cesty se dají překladači GCC sdělit pomocí přepínače -Icesta.
      // Hodí se to u složitějších projektů, které sdílejí podadresář s moduly.
      // Taky se to hodí v případech, kdy je standardní knihovna nainstalována
      // nestandardním způsobem (třeba lokálně kvůli vyzkoušení jiné verze).
      "\n"
      "- Hlavičkové soubory se vkládají proto, aby překladač mohl udělat\n"
      "  typovou kontrolu všech volání podprogramů z modulů.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Problém vícenásobného vložení")) return;
  if (pokracovat())
  {
    printh(
      "- Hlavičkový soubor se typicky využívá v různých modulech projektu.\n"
      "- Často je ho potřeba už v hlavičkovém souboru jiného modulu.\n"
      "- Často pak dojde k tomu, že je vložen vícekrát za sebou.\n"
      // Stačí příkaz #include omylem vložit zároveň do hlavičkového i zdrojového
      // souboru. Nemusí to být ani omyl - stačí vložit dva hlavičkové soubory,
      // které v sobě oba vkládají stejný hlavičkový soubor. Typicky se to stává
      // u systémových hlavičkových souborů jako stdio.h nebo stdlib.h - ty jsou
      // potřeba skoro ve všech modulech.
      "- Tím by ale došlo k opakovaným deklaracím - což je syntaktická chyba.\n"
      "\n"
      "- Řeší se to pomocí příkazů pro podmíněný překlad v hlavičkovém souboru.\n"
      "\n"
      KOD(
      "#ifndef MODUL_H_INCLUDED\n"
      "#define MODUL_H_INCLUDED\n"
      "// ... text hlavičkového souboru\n"
      "#endif // MODUL_H_INCLUDED\n"
      )
      "\n"
      "- Příkaz " KK("#ifndef") " testuje, zda není zadaný symbol definován\n"
      "  Zde je to symbol " KK("MODUL_H_INCLUDED") ".\n"
      "- Když ještě není definován, vše až po příkaz " KK("#endif") " se ponechá beze změny.\n"
      "- Když symbol už definován je, vše následující se nevloží.\n"
      "- Při prvním vložení se tento symbol definuje pomocí " KK("#define") ".\n"
      "- Název symbolu je potřeba volit tak, aby se náhodně neshodoval s nějakou\n"
      "  systémovou nebo případnou budoucí symbolickou konstantou.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Symbolické konstanty")) return;
  if (pokracovat())
  {
    #define SPATNY_VYRAZ 11+22
    #define SPRAVNY_VYRAZ (11+22)

    printh(
      "- Definují se pomocí příkazu preprocesoru " KK("#define") ".\n"
//      "- Manuál jazyka C je označuje jako object-like macros.\n"
      "- Jejich nevýhodou je, že umožňují jen omezenou typovou kontrolu.\n"
      // Například konstantní literál 12345 může být použit jako int, unsigned
      // int, long, short atd., aniž by to bylo předem specifikováno. To může
      // být v některých situacích výhoda, ale také to může způsobovat problémy.
      // Šlo by to řešit pomocí specifikace výrazu s přetypovacím operátorem,
      // ale kdo to dělá...
      // #define KRATKE_CISLO ((short) 12345)
      "\n"
      KOD(
      "#define SYMBOLICKA_KONSTANTA 12345\n"
      )
      "\n"
      "- Preprocesor pak ve zdrojovém textu nahradí každý výskyt názvu\n"
      "  " KK("SYMBOLICKA_KONSTANTA") " textem 12345.\n"
      "- Pozor! Častou chybou je psát sem středník.\n"
      "\n"
      "- Opatrně s výrazy!\n"
      "- Jde to, ale musí být uzavřeny v závorkách.\n"
      "\n"
      KOD(
      "#define SPATNY_VYRAZ 11+22\n"
      "#define SPRAVNY_VYRAZ (11+22)\n"
      )
      "\n"
      "- Důvodem je použití v podvýrazech.\n"
      "\n"
      "  => 2 * SPATNY_VYRAZ * 3 => 2 * 11+22 * 3 => %d\n"
      "  => 2 * SPRAVNY_VYRAZ * 3 => 2 * (11+22) * 3 => %d\n"
      "\n"
      , 2 * SPATNY_VYRAZ * 3
      , 2 * SPRAVNY_VYRAZ * 3
    );
  }

  if (navratNeboSekce("Konstantní globální proměnné")) return;
  if (pokracovat())
  {
    printh(
      "- Vytváří se jako proměnná, ale klíčové slovo " KK("const") " zajistí, že\n"
      "  překladač zabrání pokusům o změnu hodnoty už při překladu.\n"
      "- Tyto konstanty umožňují typovou kontrolu, jsou tedy bezpečnější.\n"
      "- Existují však i situace, kdy použití konstantní proměnné dělá potíže.\n"
      "  Viz kapitolu Pole, speciálně vícedimenzionální pole.\n"
      "\n"
      "- V hlavičkovém souboru smí být jen deklarace, což se dělá pomocí klíčového\n"
      "  slova " KK("extern") ".\n"
      "\n"
      KOD(
      "extern const int KONSTANTNI_PROMENNA;\n"
      )
      "\n"
      "- Definice (výroba) a inicializace se provede až ve zdrojovém souboru.\n"
      "\n"
      "- Důvodem pro toto je používání hlavičkového souboru ve vícero modulech.\n"
      "- Pokud by byla proměnná vyrobena ve více modulech, program by nešel\n"
      "  slinkovat. Linker by našel stejnou proměnnou vyrobenou na více místech\n"
      "  a nevěděl by, kterou má použít pro výsledný program.\n"
      "- Podmíněný překlad tento problém nevyřeší.\n"
      "\n"
      "- Stejný problém nastane, když hlavičkový soubor bude obsahovat kód, tedy\n"
      "  tělo nějaké funkce.\n"
      "\n"
    );
  }


  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void zdrojoveSoubory(void)
{
  zacatekKapitoly("Zdrojové soubory");

  printh(
    "- Zdrojový soubor modulu vypadá stejně jako zdrojový soubor programu.\n"
    "- Pouze jeden modul ze všech, co tvoří program smí obsahovat funkci " KK("main") ".\n"
    "- Pokud žádný modul funkci " KK("main") " neobsahuje, dá se z modulů vytvořit\n"
    "  knihovna funkcí.\n"
    "\n"
  );

  if (navratNeboSekce("Zdrojový soubor")) return;
  if (pokracovat())
  {
    printh(
      "- Musí mít vloženo své rozhraní a rozhraní modulů a knihoven,\n"
      "  jejichž funkce a typy používá.\n"
      "\n"
      "- Obsahuje\n"
      "  - implementace funkcí deklarovaných v hlavičkovém souboru,\n"
      "  - implementace skrytých pomocných funkcí modulu,\n"
      "  - definice a inicializace případných globálních proměnných.\n"
      "\n"
      "- Není dobré sem psát dokumentační komentáře veřejných funkcí.\n"
      "- Ty mají být v hlavičkovém souboru.\n"
      "- Jejich zdvojování vede k nekonzistencím.\n"
//      "- Dnešní IDE umí dokumentační komentáře zobrazovat ve formě bublinové\n"
//      "  nápovědy.\n"
      "- Dokumentovány by však měly být všechny neveřejné funkce a typy.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka - modul.c")) return;
  if (pokracovat())
  {
    printh(
      "- Prostuduj i samotný zdrojový text souboru modul.c.\n"
      "- Obsahuje vysvětlující komentáře.\n"
      "\n"
      KOD(
      "#include \"modul.h\"\n"
      "#include <time.h>\n"
      "\n"
      "int _aktualniRok(void)\n"
      "{\n"
      "  // ... viz soubor modul.c\n"
      "}\n"
      "\n"
      "const int KONSTANTNI_PROMENNA = 666;\n"
      "\n"
      "Tosoba vyrobOsobu(const char *jmeno, int rokNarozeni)\n"
      "{\n"
      "  // ... viz soubor modul.c\n"
      "}\n"
      "\n"
      "void tiskniOsobu(FILE * f, Tosoba o)\n"
      "{\n"
      "  // ... viz soubor modul.c\n"
      "}\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Používání rozhraní jiných modulů")) return;
  if (pokracovat())
  {
    printh(
      "- Na začátku je nutné vložit rozhraní samotného modulu.\n"
      "- Dělá se to kvůli typové kontrole prototypů a implementovaných funkcí.\n"
      "\n"
      "- Zároveň je potřeba vložit rozhraní ostatních používaných modulů a knihoven.\n"
      "- Pokud už bylo rozhraní jiného modulu nataženo v hlavičkovém souboru,\n"
      "  je zbytečné natahovat ho znovu.\n"
      "\n"
      "- Pokud překladač při překladu nezná tvar používané funkce, nemůže správně\n"
      "  zkontrolovat, zda jsou použity správně argumenty a návratová hodnota.\n"
      "- Překladač pak píše varování a předpokládá, že funkce vrací hodnotu\n"
      "  typu " KK("int") " a má proměnný počet parametrů.\n"
      "- Důsledkem toho pak může být chybný výsledný kód.\n"
      // Poznámka: Jednou z kritik jazyků C a C++ je malá bezpečnost této praxe.
      // U kritických projektů je vhodné přepnout překladač parametrem -Werror
      // do módu, kdy každé varování považuje za chybu.
      //
      // Další nebezpečnou vlastností céčkových modulů je, že mohou využívat
      // i skryté funkce jiných modulů, pokud znají jejich jména a rozhraní.
      // To se dá zjistit buďto ze zapomenutých ladících informací nebo
      // odhadnout hackováním.
      //
      // Používání skrytých funkcí mimo modul je velice nerozumné počínání,
      // protože tyto funkce obvykle nejsou navrženy jako bezpečné,
      // tj. nehlídají meze parametrů, předpokládají použití v určitém kontextu
      // a podobně. Jejich autoři prostě nepředpokládali, že by je mohl využívat
      // někdo jiný a může to být zdrojem chyb nebo i bezpečnostních hrozeb.
      // Druhé nebezpečí spočívá v tom, že po aktualizaci knihovny mohou skryté
      // funkce zmizet nebo úplně změnit rozhraní, protože autoři nejsou vázáni
      // žádným zveřejněným rozhraním k jejich zpětné kompatibilitě.
      "\n"
    );
  }

  if (navratNeboSekce("Implementace funkcí")) return;
  if (pokracovat())
  {
    printh(
      "- U veřejných funkcí se musí dodržet stejná hlavička, jako v hlavičkovém\n"
      "  souboru.\n"
      "- Pokud by funkce měla omylem jinou hlavičku, bude se považovat za\n"
      "  vnitřní skrytou funkci modulu.\n"
      "- Pozor! Dokud nedojde k použití veřejné funkce, nebude překladač\n"
      "  neimplementovaný prototyp z hlavičkového souboru považovat za chybu.\n"
      "=> Je velmi důležité všechny implementované funkce otestovat, tedy\n"
      "   skutečně je zavolat.\n"
      "\n"
      "- Pro implementaci veřejné funkce se obvykle používají i vnitřní,\n"
      "  neveřejné funkce modulu.\n"
      "- Veřejné funkce by měly pečlivě kontrolovat hodnoty zpracovávaných\n"
      "  parametrů (správné rozsahy, " KK("NULL") " apod.).\n"
      "\n"
      "- Konvence: Neveřejné funkce mají název začínající podtržítkem (_).\n"
      "\n"
    );
  }

  if (navratNeboSekce("Inicializace konstantních proměnných")) return;
  if (pokracovat())
  {
    printh(
      "- Veřejné globální konstantní proměnné se deklarují pomocí klíčového slova\n"
      "  " KK("extern") " v hlavičkovém souboru.\n"
      "- Při definici ve zdrojovém souboru se toto klíčové slovo vynechává.\n"
      "\n"
      KOD(
      "const int KONSTANTNI_PROMENNA = 666;\n"
      )
      "\n"
      "- Hodnotu jde inicializovat pouze konstantní proměnnou.\n"
      "- Nejde sem přiřadit výsledek volání funkce.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}


////////////////////////////////////////////////////////////////////////////////
void prekladProgramuSModuly(void)
{
  zacatekKapitoly("Překlad programů s moduly");

  printh(
    "- Každé vývojové prostředí (IDE) řeší překlad projektu s moduly samo.\n"
    // IDE samo nepřekládá, ale volá překladač s vhodnými parametry
    // zajišťuje překlad a linkování modulů překladačem odděleně ve správném
    // pořadí.
    "- Programátor se zde nemusí o nic starat.\n"
    "\n"
    "- Programátorské editory (VSCode, PSPad, vim apod.) samy překládat neumí,\n"
    "  ale často využívají program " KK("make") ". Skript " KK("Makefile") " pro překlad si\n"
    "  uživatel musí upravit sám.\n"
    "\n"
    "- Ruční volání překladače z příkazového řádku se skoro nepoužívá, protože\n"
    "  je to pracné.\n"
    "- Je ale dobré vědět jak na to, abychom byli schopni vytvářet " KK("Makefile") ",\n"
    "  nebo upravit chování IDE kvůli specifickým požadavkům.\n"
    // Profesionální IDE umožňují pokročilým uživatelům používat vlastní Makefile.
    // Pokud je potřeba překládat nějaké moduly specifickým způsobem je to
    // lepší a přenositelnější řešení než složitě nastavovat IDE samotné.
    // Code::Blocks to umí pomocí Project -> Properties -> Project settings.
    "\n"
    "- V následujících ukázkách je použit překladač GCC.\n"
    "- Jiné překladače se mohou lišit v detailech, ale v principu fungují podobně.\n"
    "\n"
  );

  if (navratNeboSekce("Co se děje při překladu programu")) return;
  if (pokracovat())
  {
    printh(
      "- Překlad programu se skládá z několika kroků.\n"
      "- Překladač sám volá různé programy pro různé fáze překladu.\n"
      "- Tyto fáze lze provádět i samostatně - viz dále.\n"
      "\n"
      KOD(
      "+-----------+    +-----------+                              +-------------+\n"
      "|preprocesor|    | kompilace |    +-----+    +---------+    | spustitelný |\n"
      "|    *.c    | -> |  modulů   | -> | *.o | -> |linkování| -> |   binární   |\n"
      "|    *.h    |    |    *.c    |    +-----+    +---------+    |   soubor    |\n"
      "+-----------+    +-----------+                              +-------------+\n"
      )
      "\n"
      PODSEKCE("Preprocesor")
      "- Dělá textové náhrady ve zdrojovém textu (příkazy " KK("#include") ", " KK("#define") " aj.).\n"
      "- Také nahradí každý komentář právě jednou mezerou.\n"
      "- Výsledkem jsou čisté zdrojové soubory (" KK(".c") ").\n"
      "\n"
      PODSEKCE("Kompilátor")
      "- Překládá zdrojové soubory (" KK(".c") ") jednotlivých modulů do binárních\n"
      "  objektových souborů (" KK(".o") ").\n"
      "- Objektový soubor je binárka, v níž jsou na místech volání funkcí z jiných\n"
      "  modulů a knihoven pouze značky pro linker.\n"
//      "- Některé překladače mohou provádět ještě mezikrok, kdy se nejprve\n"
//      "  jazyk C přeloží do jazyka symbolických instrukcí a ten se pak překládá\n"
//      "  pomocí programu assembler do objektových souborů.\n"
//      "  Dělá se to proto, že assembler může provádět snadněji optimalizaci kódu.\n"
      "\n"
      PODSEKCE("Linker")
      "- Provádí linkování kódu, tedy spojení objektových souborů do jediného\n"
      "  binárního souboru.\n"
      "- Do všech míst, kde se volají funkce z jiných modulů, dosadí skutečné\n"
      "  adresy funkcí.\n"
      "- V případě použití dynamických knihoven dosadí kód pro zavolání takové\n"
      "  funkce.\n"
      "- Výsledkem je spustitelný binární soubor (např. " KK(".exe") ").\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ruční překlad malého projektu")) return;
  if (pokracovat())
  {
    printh(
      "- Malé projekty umí překladač přeložit sám.\n"
      "- Sám pak volá preprocesor i linker ve správném pořadí.\n"
      "- Moduly by neměly mít křížové závislosti.\n"
      "\n"
      KOD(
      "  gcc -std=c99 -pedantic -Wall -Wextra modul1.c modul2.c main.c -o program\n"
      )
      "\n"
      "- V tomto případě se projekt skládá ze 3 modulů: " KK("modul1.c") ", " KK("modul2.c") "\n"
      "  a " KK("main.c") ".\n"
      "- Pořadí modulů je dáno jejich vzájemnými závislostmi.\n"
      "- Hlavičkové soubory se neuvádějí. Preprocesor si je najde sám.\n"
      "- Výsledkem bude spustitelný binární soubor " KK("program") " v unixu, nebo\n"
      "  " KK("program.exe") " ve Windows.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Oddělený překlad modulů")) return;
  if (pokracovat())
  {
    printh(
      "- Toto je užitečné, když je modulů hodně anebo mají křížové závislosti.\n"
      "- Takto se to obvykle dělá v " KK("Makefile") ".\n"
      "\n"
      KOD(
      "  gcc -std=c99 -pedantic -Wall -Wextra -c modul1.c -o modul1.o\n"
      "  gcc -std=c99 -pedantic -Wall -Wextra -c modul2.c -o modul2.o\n"
      "  gcc -std=c99 -pedantic -Wall -Wextra -c main.c -o main.o\n"
      "  gcc modul1.o modul2.o main.o -o program\n"
      )
      "\n"
      "- Přepínač " KK("-c") " říká, že se má udělat pouze překlad, ne linkování.\n"
      "- V posledním kroku překladač sám pozná, že na vstupu jsou objektové\n"
      "  soubory a odvodí si, že má pouze zavolat linker.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Program make a Makefile")) return;
  if (pokracovat())
  {
    printh(
      "- Program " KK("make") " se používá pro překlad rozsáhlejších projektů s moduly.\n"
      "- Je-li v adresáři projektu umístěn soubor " KK("Makefile") ", pak stačí v tomto\n"
      "  adresáři spustit program " KK("make") " (i bez parametrů) a projekt se začne\n"
      "  překládat podle pravidel zapsaných v souboru " KK("Makefile") ".\n"
      "\n"
      "- Návod k vytváření souboru " KK("Makefile") " a popis fungování programu " KK("make") "\n"
      "  je na adrese https://jaknaprojekty.davidm.cz.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void coJeToKnihovna(void)
{
  zacatekKapitoly("Co je to knihovna");

  printh(
    "- Je to princip modulu povýšený o úroveň výše.\n"
    "- Obvykle je složena z více modulů.\n"
    "- Modul si programátor píše pro své vlastní projekty.\n"
    "- Knihovna je software určený pro ostatní programátory a uživatele.\n"
    "\n"
    "- Obsahuje funkce řešící celou třídu příbuzných problémů, například GUI,\n"
    "  komunikaci po síti, grafický výstup apod.\n"
    "- Obsahuje přeložený binární kód obecně použitelných podprogramů.\n"
    "- Uživatel - programátor potřebuje znát jen její rozhraní (v C hlavičkové\n"
    "  soubory), aby ji mohl používat.\n"
    "\n"
    "- Rozlišují se statické a dynamické knihovny.\n"
    "\n"
  );

  if (navratNeboSekce("Statické knihovny")) return;
  if (pokracovat())
  {
    printh(
      "- Staticky linkované knihovny fungují podobně jako moduly.\n"
      "- Jsou to soubory s koncovkou " KK(".a") ".\n"
      "- Využívají se pouze v době překladu nových programů.\n"
      "- V podstatě je to zásobárna přeložených modulů.\n"
      "\n"
    PODSEKCE("Výhody a nevýhody")
      "+ Šíří se jako přeložená binárka spolu s veřejnými hlavičkovými soubory.\n"
      "+ Instaluje si ji pouze vývojář.\n"
      "  => Koncový uživatel hotového programu ji vůbec nepotřebuje.\n"
      "\n"
      "- Výsledný spustitelný soubor může být objemný - binární kód se kopíruje do\n"
      "  výsledné binárky.\n"
      "\n"
      "+- Nezávislost na změnách verzí použité knihovny.\n"
      "  - Opravy chyb v knihovně => program se musí znovu přeložit.\n"
      "  + Změna rozhraní knihovny => nevadí, program funguje s tím, co bylo použito\n"
      "    v době překladu.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Dynamické knihovny")) return;
  if (pokracovat())
  {
    printh(
      "- Soubory s koncovkou " KK(".so") " v unixu, nebo " KK(".dll") " ve Windows\n"
      "- Dynamické proto, že se natahují až za běhu.\n"
      "- Spustitelný program v místě volání knihovní funkce žádá operační systém.\n"
      "  Ten do paměti natáhne žádanou dynamickou knihovnu a teprve pak zavolá\n"
      "  žádanou funkci.\n"
      "- Při prvním použití je to pomalejší, další spouštění už je rychlé.\n"
      "- Při kompilaci se do výsledné binárky nekopíruje kód knihovní funkce, ale pouze\n"
      "  název funkce a dynamické knihovny. Tyto informace následně používá OS v době\n"
      "  spuštění programu a volání konkrétních funkcí.\n"
      "\n"
    PODSEKCE("Výhody")
      "+ Menší velikosti spustitelných souborů, protože programy sdílí stejný kód\n"
      "  umístěný v knihovnách.\n"
      "+ Snazší opravy chyb - stačí nainstalovat opravenou verzi knihovny a všechny\n"
      "  programy, které ji využívají začnou fungovat lépe.\n"
      // Pokud na té chybě nezávisí jejich fungování - ale to je problém špatného
      // návrhu programů.
      "\n"
    PODSEKCE("Nevýhody")
//      "- Musí je podporovat operační systém.\n"
//      To dnes asi není problém, protože všechny operační systémy to dávno podporují.
      "- V systému ji musí mít nainstalovánu nejen vývojář, ale i koncový uživatel.\n"
      "- Oba musí mít nainstalovánu správnou (kompatibilní) verzi knihovny.\n"
      "- Dochází-li často ke změnám rozhraní knihoven, je uživatel nucen mít\n"
      "  nainstalováno několik verzí stejné knihovny.\n"
      // Například proto, že některé programy používají ještě to starší rozhraní,
      // kdežto novější programy už přešly na novější verzi knihovny.
      "\n"
    );
  }

  konecKapitoly(__func__);
}


////////////////////////////////////////////////////////////////////////////////
void prekladKnihovny(void)
{

  zacatekKapitoly("Překlad a použití knihovny");

  printh(
    "- U statické a dynamické knihovny se postup liší.\n"
    "- Odlišnosti jsou i v různých operačních systémech.\n"
    "- Knihovny se nevytváří linkerem, ale jinými programy.\n"
    "\n"
    "- Použití knihovny naproti tomu vypadá vždy stejně.\n"
    "\n"
  );

  if (navratNeboSekce("Překlad programu využívajícího knihovnu")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "gcc -o program modul1.o main.o -lmojeknihovna\n"
      )
      "\n"
      "- Knihovna musí být nainstalovaná v systému.\n"
      "- Statická knihovna je soubor s názvem " KK("libmojeknihovna.a") " v unixu nebo\n"
      "  " KK("mojeknihovna.lib") " ve Windows.\n"
      "- Dynamická knihovna se jmenuje " KK("libmojeknihovna.so") " v unixu\n"
      "  nebo " KK("mojeknihovna.dll") " ve Windows.\n"
      "- Spolu s knihovnou musí být nainstalovány její hlavičkové soubory.\n"
      "- Parametry pro linkování knihoven se uvádí až jako úplně poslední.\n"
      "\n"
      "- Je-li knihovna na neobvyklém místě, použije se přepínač " KK("-L") ".\n"
      "- Jsou-li hlavičkové soubory na neobvyklém místě, použije se přepínač " KK("-I") "\n"
      "\n"
      KOD(
      "gcc [další přepínače] -I/cesta/k/rozhrani -c modul1.c -o modul1.o\n"
      "gcc [další přepínače] -I/cesta/k/rozhrani -c main.c -o main.o\n"
      "gcc -o program modul1.o main.o -L/cesta/ke/knihovne -lmojeknihovna\n"
      )
      "\n"
      "- Parametry " KK("-I") " a " KK("-L") " nejsou potřeba, když je knihovna i rozhraní přímo\n"
      "  v adresáři překládaného projektu.\n"
      "\n"
      "- Dynamickou knihovnu musí mít uživatel nainstalovánu na obvyklém místě\n"
      "  svého operačního systému.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Překlad statické knihovny v Linuxu")) return;
  if (pokracovat())
  {
    printh(
      "- Nejprve se přeloží moduly obvyklým způsobem.\n"
      "- Poté se pro vytvoření knihovny použije program " BB("ar") ".\n"
      "\n"
      KOD(
      "gcc [další přepínače] -c modul1.c -o modul1.o\n"
      "gcc [další přepínače] -c modul2.c -o modul2.o\n"
      "ar rcs libmojeknihovna.a modul1.o modul2.o\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Překlad statické knihovny ve Windows")) return;
  if (pokracovat())
  {
    printh(
      "- Nejprve se přeloží moduly obvyklým způsobem.\n"
      "- I když se ve Windows obvykle používá koncovka " KK(".lib") ", je formát\n"
      "  stejný jako u unixových knihoven s koncovkou " KK(".a") ".\n"
      "  => Jde použít program " BB("ar") " stejně jako v Linuxu (například z MINGW).\n"
      "- Windows mají i svůj vlastní program na tvorbu knihoven nazvaný " BB("lib") ".\n"
      "\n"
      KOD(
      "gcc [další přepínače] -c modul1.c -o modul1.o\n"
      "gcc [další přepínače] -c modul2.c -o modul2.o\n"
      "lib /OUT:mojeknihovna.lib modul1.o modul2.o\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Překlad dynamické knihovny v Linuxu")) return;
  if (pokracovat())
  {
    printh(
      "- Při překladu modulů se používá přepínač " KK("-fPIC") ", který vytváří\n"
      "  tzv. pozičně nezávislý kód.\n"
      // Position independent code: Používá relativní adresy funkcí místo
      // absolutních adres, což je výhodné u dynamických knihoven, protože to
      // umožňuje operačnímu systému různě přehazovat umístění knihovny
      // paměti za běhu a sdílení jejího kódu různými programy.
      // Možná to ale v novějších verzích nebude potřeba...
      "- Dynamickou knihovnu vyrábí překladač GCC použitý s přepínačem " KK("-shared") ".\n"
      "\n"
      KOD(
      "gcc [další přepínače] -fPIC -c modul1.c -o modul1.o\n"
      "gcc [další přepínače] -fPIC -c modul2.c -o modul2.o\n"
      "gcc -shared modul1.o modul2.o -o libmojeknihovna.so\n"
      "\n"
      )
    );
  }

  if (navratNeboSekce("Překlad dynamické knihovny ve Windows")) return;
  if (pokracovat())
  {
    printh(
      "- Pokud bude knihovna používána jen v projektech používajících MinGW,\n"
      // MinGW - Implementace GCC pro Windows.
      "  vytváří se a používá podobně jako v Linuxu.\n"
      "\n"
      KOD(
      "gcc [další přepínače] -c modul1.c -o modul1.o\n"
      "gcc [další přepínače] -c modul2.c -o modul2.o\n"
      // TODO: Zkontrolovat, zda ve Windows přepínač -fPIC funguje nebo ne.
      // Na webu jsou rozporné informace.
      "gcc -shared modul1.o modul2.o -o mojeknihovna.dll\n"
      )
      "\n"
      "- Pokud má být DLL knihovna kompatibilní s knihovnami překladače MSVC,\n"
      "  musí se spolu s DLL knihovnou vyrobit ještě tzv. importní knihovna, což\n"
      "  je statická knihovna, která ale obsahuje cestu k DLL knihovně a seznam\n"
      "  jejích veřejných symbolů. Neobsahuje žádný kód.\n"
      "- Dá se vyrobit pomocí nástroje " BB("dlltool") ", nebo přímo pomocí " BB("gcc") ":\n"
      "\n"
      KOD(
      "gcc [další přepínače] -c modul1.c -o modul1.o\n"
      "gcc [další přepínače] -c modul2.c -o modul2.o\n"
      "gcc -shared modul1.o modul2.o -o mojeknihovna.dll -Wl,--out-implib,mojeknihovna.dll.lib\n"
      )
      // TODO: Zkontrolovat ve Windows, jestli není potřeba ještě něco.
      "\n"
      "- Za přepínačem " KK("-Wl") " nejsou žádné mezery. Čárkami jsou odděleny parametry\n"
      "  pro linker.\n"
      "- Vzniknou tak dva soubory: " KK("mojeknihovna.dll") " a " KK("mojeknihovna.dll.lib") ".\n"
      "- Importní knihovna " KK("mojeknihovna.dll.lib") " se také může jmenovat\n"
      "  " KK("mojeknihovna.dll.a") ".\n"
      "- Program se linkuje právě s ní, ne přímo s DLL knihovnou.\n"
      "\n"
      KOD(
      "gcc main.o -o main.exe -L. -lmojeknihovna.dll\n"
      )
      "\n"
    );
  }

  konecKapitoly(__func__);
}

