/* Hlavičkový soubor modulu modul
 *
 * Autor: David Martinek, 2023
 *
 * 2023-08-10-13.59
 */

// Ošetření proti vícenásobnému vložení hlavičkového souboru
// pomocí podmíněného překladu
#ifndef MODUL_H_INCLUDED
#define MODUL_H_INCLUDED

// Rozhraní stdio.h je potřeba vložit kvůli použití typu FILE*
// ve funkci tiskniOsobu.
#include <stdio.h>

/**
 * Symbolická konstanta. Anglicky object-like macro.
 * Pozor, zapisovat bez středníku (;)!
 */
#define SYMBOLICKA_KONSTANTA 12345

/**
 * Deklarace (extern) konstantní (const) globální proměnné
 * Definice a inicializace je ve zdrojovém souboru modul.c.
 * Tady středník je!
 */
extern const int KONSTANTNI_PROMENNA;

/** Deklarace veřejného typu Tosoba */
typedef struct _osoba
{
  const char * jmeno; /**< Položka jméno je ukazatel na konstatní string. */
  unsigned int vek;   /**< Věk osoby je neznaménkové číslo. */
} Tosoba;


// Následují veřejné prototypy funkcí i s dokumentačními komentáři.

/** Vyrobí osobu ze zadaným jménem a dopočítá věk podle roku narození.
 *
 * \param jmeno const char* Jméno osoby musí být konstantní string.
 * \param rokNarozeni int   Rok narození osoby.
 * \return Tosoba Vrací strukturu vytvořenou podle zadaných údajů.
 */
Tosoba vyrobOsobu(const char *jmeno, int rokNarozeni);

/** Vytiskne zadanou osobu na zadaný výstup.
 *
 * \param f FILE* Otevřený výstupní soubor nebo datový proud.
 * \param o Tosoba Osoba, která se bude tisknou.
 */
void tiskniOsobu(FILE * f, Tosoba o);

// konec podmíněného překladu
#endif // MODUL_H_INCLUDED
